/*
 * Copyright 2000-2001 by Jean-Claude Christophe <jch@oleane.net>
 *                                               <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * encrypt.c  This file contains functions for encrypt packets
 */

#include <sys/types.h>

#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>

#include "md5.h"
#include "draft2.h"

void
create_md5_hash(session_id, key, version, seq_no, prev_hash, hash)
    int session_id;
    char *key;
    u_char version;
    u_char seq_no;
    u_char *prev_hash;
    u_char *hash;
{
    u_char *md_stream, *mdp;
    int md_len;
    MD5_CTX mdcontext;
    
    md_len = sizeof(session_id) + strlen(key) + sizeof(version) + sizeof(seq_no);
    
    if (prev_hash)
	md_len += MD5_LEN;
    
    mdp = md_stream = (u_char *) malloc(md_len);
    bcopy(&session_id, mdp, sizeof(session_id));
    mdp += sizeof(session_id);
    
    bcopy(key, mdp, strlen(key));
    mdp += strlen(key);
    
    bcopy(&version, mdp, sizeof(version));
    mdp += sizeof(version);
    
    bcopy(&seq_no, mdp, sizeof(seq_no));
    mdp += sizeof(seq_no);
    
    if (prev_hash) {
	bcopy(prev_hash, mdp, MD5_LEN);
	mdp += MD5_LEN;
    }
    MD5Init(&mdcontext);
    MD5Update(&mdcontext, md_stream, md_len);
    MD5Final(hash, &mdcontext);
    free(md_stream);
    return;
}

/*
 * return 1 on success, -1 on failure
 */
int
md5_xor(myhead, data, secret)
    t_pkt_hdr *myhead;
    u_char *data;
    char *secret;
{
    int i, j;
    u_char hash[MD5_LEN];	/* the md5 hash */
    u_char last_hash[MD5_LEN];	/* the last hash we generated */
    u_char *prev_hashp = (u_char *) NULL;	/* pointer to last created
						 * hash */
    int data_len;
    int session_id;
    u_char version;
    u_char seq_no;
    
    if (!secret)
	return(-1);
    
    data_len = ntohl(myhead->length);
    session_id = myhead->session_id; /* always in network order for hashing (no ntoh !) */
    version = myhead->version;
    seq_no = myhead->seq_no;
    
    for (i = 0; i < data_len; i += MD5_LEN) {
	create_md5_hash(session_id, secret, version, seq_no, prev_hashp, hash);
	bcopy(hash, last_hash, MD5_LEN);
	prev_hashp = last_hash;
	for (j = 0; j < MD5_LEN; j++) {
	    if ((i + j) >= data_len) {
		/* on bascule le flag */
		myhead->flags = (myhead->flags == TAC_PLUS_UNENCRYPTED_FLAG) 
		    ? TAC_PLUS_ENCRYPTED_FLAG : TAC_PLUS_UNENCRYPTED_FLAG;
		return(1);
	    }
	    data[i + j] ^= hash[j];
	}
    }
    /* on bascule le flag */
    myhead->flags = (myhead->flags == TAC_PLUS_UNENCRYPTED_FLAG)
	? TAC_PLUS_ENCRYPTED_FLAG : TAC_PLUS_UNENCRYPTED_FLAG;
    return(1);
}
